package org.richfaces.schedule;

import org.ajax4jsf.model.DataVisitor;
import org.ajax4jsf.model.ExtendedDataModel;
import org.ajax4jsf.model.Range;

import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import java.io.IOException;
import java.util.*;
import org.richfaces.component.UISchedule;
import org.richfaces.component.events.ScheduleItemMovedEvent;
import org.richfaces.component.events.ScheduleItemResizedEvent;
import org.richfaces.component.events.ScheduleItemSelectedEvent;
import org.richfaces.component.events.ScheduleViewChangedEvent;
import org.richfaces.component.model.DateRange;

public class Bean {

    private String text;
    private boolean allowTaskMoving;
    private List<ScheduleTask> allTasks = new ArrayList<ScheduleTask>();
    private ExtendedDataModel lazyDataModel = new MyDataModel();
    private String selectedEventId;
    private int firstDay = Calendar.SUNDAY;
    private boolean showWeekends;
    private Date initialDate;
    private boolean allowTaskResizing;
    private String weekMode = UISchedule.WEEK_MODE_FIXED;
    private String allDayText = "All day";
    private boolean allDayByDefault;
    private boolean allDaySlot = true;
    private Double aspectRatio = 1.;
    private String axisFormat = "h(:mm)tt";
    private Integer defaultEventMinutes = 90;
    private Integer contentHeight = 400;
    private Double dragOpacity = .2;
    private Integer dragRevertDuration = 2000;
    private Boolean editable = true;
    private Integer firstHour = 8;
    private String headerCenter = "title";
    private String headerLeft = "prevYear,nextYear";
    private String headerRight = "agendaDay,agendaWeek,month today prev,next";
    private Integer height = 400;
    private boolean isRTL;
    private Integer maxTime = 17;
    private Integer minTime = 8;
    private Integer slotMinutes = 30;
    private String view = UISchedule.VIEW_MONTH;

    public Bean() {
        Calendar instance = Calendar.getInstance();
        instance.setTime(getInitialDate());
        for (int i = -30; i < 60; i++) {
            instance.setTime(getInitialDate());
            instance.add(Calendar.DAY_OF_YEAR, i);
            allTasks.add(new ScheduleTask("" + i, "Title " + i, instance.getTime(), instance.getTime(), true));
        }
    }

    public Boolean taskMoved(ScheduleItemMovedEvent event) {
        System.out.println("taskMoved invoked " + event + " : " + isAllowTaskMoving());
        if (isAllowTaskMoving()) {
            FacesContext.getCurrentInstance().addMessage(null, new FacesMessage("Item moved", event.toString()));
            ScheduleTask task = getTask(event.getEventId());
            selectedEventId = event.getEventId();
            if (task != null) {
                boolean endDateEqualsStartDate = task.getStartDate().equals(task.getEndDate());
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(task.getStartDate());
                calendar.add(Calendar.DAY_OF_MONTH, event.getDayDelta());
                calendar.add(Calendar.MINUTE, event.getMinuteDelta());
                task.setStartDate(calendar.getTime());
                if (!event.isAllDay() && endDateEqualsStartDate) {
                    calendar.setTime(task.getStartDate());
                    calendar.add(Calendar.MINUTE, getDefaultEventMinutes());
                } else {
                    calendar.setTime(task.getEndDate());
                    calendar.add(Calendar.DAY_OF_MONTH, event.getDayDelta());
                    calendar.add(Calendar.MINUTE, event.getMinuteDelta());
                }
                task.setEndDate(calendar.getTime());
                task.setAllDay(event.isAllDay());
            }
        } else {
            FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, "It is not allowed to move this item", event.toString()));
        }
        return isAllowTaskMoving();
    }

    public Boolean taskResized(ScheduleItemResizedEvent event) {
        System.out.println("taskResized invoked " + event + " : " + isAllowTaskResizing());
        if (isAllowTaskResizing()) {
            FacesContext.getCurrentInstance().addMessage(null, new FacesMessage("Item resized", event.toString()));
            ScheduleTask task = getTask(event.getEventId());
            selectedEventId = event.getEventId();
            if (task != null) {
                Calendar calendar = Calendar.getInstance();
                Date date = task.getEndDate() == null ? task.getStartDate() : task.getEndDate();
                calendar.setTime(date);
                calendar.add(Calendar.DAY_OF_MONTH, event.getDayDelta());
                calendar.add(Calendar.MINUTE, event.getMinuteDelta());
                task.setEndDate(calendar.getTime());
            }
        } else {
            FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_ERROR, "It is not allowed to resize this item", event.toString()));
        }
        return isAllowTaskResizing();
    }

    public void viewChanged(ScheduleViewChangedEvent event) {
        System.out.println("viewChanged invoked " + event);
        FacesContext.getCurrentInstance().addMessage(null, new FacesMessage("View changed", event.toString()));
        setView(event.getView());
    }

    public void taskSelected(ScheduleItemSelectedEvent event) {
        FacesContext.getCurrentInstance().addMessage(null, new FacesMessage("Task selected", event.toString()));
        selectedEventId = event.getEventId();
    }

    public ScheduleTask getSelectedTask() {
        return getTask(getSelectedEventId());
    }

    protected ScheduleTask getTask(String id) {
        if (id == null) {
            return null;
        }
        for (ScheduleTask task : allTasks) {
            if (id.equals(task.getId())) {
                return task;
            }
        }
        return null;
    }

    public String getText() {
        return text;
    }

    public void setText(String text) {
        this.text = text;
    }

    public List<ScheduleTask> getAllEvents() {
        return allTasks;
    }

    public ExtendedDataModel getLazyDataModel() {
        return lazyDataModel;
    }

    public boolean isAllowTaskMoving() {
        return allowTaskMoving;
    }

    public void setAllowTaskMoving(boolean allowTaskMoving) {
        this.allowTaskMoving = allowTaskMoving;
    }

    public boolean isAllowTaskResizing() {
        return allowTaskResizing;
    }

    public void setAllowTaskResizing(boolean allowTaskResizing) {
        this.allowTaskResizing = allowTaskResizing;
    }

    public String getSelectedEventId() {
        return selectedEventId;
    }

    public Date getInitialDate() {
        if (initialDate == null) {
            Calendar instance = Calendar.getInstance();
            instance.set(Calendar.YEAR, 2012);
            instance.set(Calendar.MONTH, 7);
            instance.set(Calendar.DATE, 22);
            initialDate = instance.getTime();
        }
        return initialDate;
    }

    public int getFirstDay() {
        return firstDay;
    }

    public void setFirstDay(int firstDay) {
        this.firstDay = firstDay;
    }

    public boolean isShowWeekends() {
        return showWeekends;
    }

    public void setShowWeekends(boolean showWeekends) {
        this.showWeekends = showWeekends;
    }

    public void setInitialDate(Date initialDate) {
        this.initialDate = initialDate;
    }

    public String getWeekMode() {
        return weekMode;
    }

    public void setWeekMode(String weekMode) {
        this.weekMode = weekMode;
    }

    public void setAllDayText(String allDayText) {
        this.allDayText = allDayText;
    }

    public String getAllDayText() {
        return allDayText;
    }

    public void setAllDayByDefault(boolean allDayByDefault) {
        this.allDayByDefault = allDayByDefault;
    }

    public boolean getAllDayByDefault() {
        return allDayByDefault;
    }

    public void setAllDaySlot(boolean allDaySlot) {
        this.allDaySlot = allDaySlot;
    }

    public boolean getAllDaySlot() {
        return allDaySlot;
    }

    public void setAspectRatio(Double aspectRatio) {
        this.aspectRatio = aspectRatio;
    }

    public Double getAspectRatio() {
        return aspectRatio;
    }

    public void setAxisFormat(String axisFormat) {
        this.axisFormat = axisFormat;
    }

    public String getAxisFormat() {
        return axisFormat;
    }

    public void setDefaultEventMinutes(Integer defaultEventMinutes) {
        this.defaultEventMinutes = defaultEventMinutes;
    }

    public Integer getDefaultEventMinutes() {
        return defaultEventMinutes;
    }

    public void setContentHeight(Integer contentHeight) {
        this.contentHeight = contentHeight;
    }

    public Integer getContentHeight() {
        return contentHeight;
    }

    public void setDragOpacity(Double dragOpacity) {
        this.dragOpacity = dragOpacity;
    }

    public Double getDragOpacity() {
        return dragOpacity;
    }

    public void setDragRevertDuration(Integer dragRevertDuration) {
        this.dragRevertDuration = dragRevertDuration;
    }

    public Integer getDragRevertDuration() {
        return dragRevertDuration;
    }

    public void setEditable(Boolean editable) {
        this.editable = editable;
    }

    public Boolean getEditable() {
        return editable;
    }

    public void setFirstHour(Integer firstHour) {
        this.firstHour = firstHour;
    }

    public Integer getFirstHour() {
        return firstHour;
    }

    public void setHeaderCenter(String headerCenter) {
        this.headerCenter = headerCenter;
    }

    public String getHeaderCenter() {
        return headerCenter;
    }

    public void setHeaderLeft(String headerLeft) {
        this.headerLeft = headerLeft;
    }

    public String getHeaderLeft() {
        return headerLeft;
    }

    public void setHeaderRight(String headerRight) {
        this.headerRight = headerRight;
    }

    public String getHeaderRight() {
        return headerRight;
    }

    public void setHeight(Integer height) {
        this.height = height;
    }

    public Integer getHeight() {
        return height;
    }

    public void setIsRTL(Boolean isRTL) {
        this.isRTL = isRTL;
    }

    public Boolean getIsRTL() {
        return isRTL;
    }

    public void setMaxTime(Integer maxTime) {
        this.maxTime = maxTime;
    }

    public Integer getMaxTime() {
        return maxTime;
    }

    public void setMinTime(Integer minTime) {
        this.minTime = minTime;
    }

    public Integer getMinTime() {
        return minTime;
    }

    public void setSlotMinutes(Integer slotMinutes) {
        this.slotMinutes = slotMinutes;
    }

    public Integer getSlotMinutes() {
        return slotMinutes;
    }

    public void setView(String view) {
        this.view = view;
    }

    public String getView() {
        return view;
    }

    private class MyDataModel extends ExtendedDataModel {

        Object rowKey;
        java.util.Map wrappedDataMap = new HashMap();
        java.util.Map indexToRowKey = new HashMap();
        java.util.Map rowKeyToIndex = new HashMap();
        int rowCount = -1;
        int rowIndex = -1;

        @Override
        public void setRowKey(Object key) {
            this.rowKey = key;
            Integer index = (Integer) rowKeyToIndex.get(key);
            if (index == null) {
                index = -1;
            }
            if (index != getRowIndex()) {
                setRowIndex(rowIndex);
            }
        }

        @Override
        public Object getRowKey() {
            return rowKey;
        }

        @Override
        public void walk(FacesContext context, DataVisitor visitor, Range range, Object argument) throws IOException {
            Date startDate = ((DateRange) range).getStartDate();
            Date endDate = ((DateRange) range).getEndDate();

//            ------------------

//            boolean showWeekends = false;
//            Calendar instance = Calendar.getInstance();
//            /**
//             * Calculate first day of week
//             */
//            instance.setTime(startDate);
//            int firstDayOfWeek = instance.get(Calendar.DAY_OF_WEEK);
//            instance.setFirstDayOfWeek(firstDayOfWeek);
//            /**
//             * Move to first of month
//             */
//            instance.add(Calendar.DAY_OF_YEAR, 9);
//            instance.set(Calendar.DATE, 1);
//            int dayOfWeekOfFirstDayInMonth = instance.get(Calendar.DAY_OF_WEEK);
//            if (!showWeekends) {
//                if (dayOfWeekOfFirstDayInMonth == Calendar.SUNDAY) {
//                    instance.add(Calendar.DATE, 1);
//                } else if (dayOfWeekOfFirstDayInMonth == Calendar.SATURDAY) {
//                    instance.add(Calendar.DATE, 2);
//                }
//            }
//            int firstWeek = instance.get(Calendar.WEEK_OF_MONTH);
//            /**
//             * Move to the end of month
//             */
//            instance.set(Calendar.DATE, instance.getActualMaximum(Calendar.DATE));
//            int dayOfWeekOfLastDayInMonth = instance.get(Calendar.DAY_OF_WEEK);
//            if (!showWeekends) {
//                if (dayOfWeekOfLastDayInMonth == Calendar.SUNDAY) {
//                    instance.add(Calendar.DATE, -2);
//                } else if (dayOfWeekOfLastDayInMonth == Calendar.SATURDAY) {
//                    instance.add(Calendar.DATE, -1);
//                }
//            }
//            int lastWeek = instance.get(Calendar.WEEK_OF_MONTH);
//            int weeksCount = lastWeek - firstWeek + 1;
//            System.out.println("start=" + startDate + ";end=" + endDate + ";util=" + instance.getTime());
//            System.out.println("weeks=" + weeksCount + ";1dayOfWeek=" + dayOfWeekOfFirstDayInMonth + ";2dayOfWeek=" + dayOfWeekOfLastDayInMonth + ";firstWeek=" + firstWeek + ";lastWeek=" + lastWeek);
//            /**
//             * Move to last displayedDate
//             */
//            int displayedDays = showWeekends ? 7 : 5;
////            displayedDays-dayOfWeekOfFirstDisplayedDay;
//            boolean isFixed = true;
//            if (isFixed) {
//                int daysToAdd = 0;
//                while (weeksCount < 6) {
//                    daysToAdd += 7;
//                    weeksCount++;
//                }
//                instance.add(Calendar.DAY_OF_YEAR, daysToAdd);
//            }
//            System.out.println("last date in range=" + instance.getTime());
//            instance.setMinimalDaysInFirstWeek(1);
//            System.out.println(instance.getActualMinimum(Calendar.WEEK_OF_MONTH));
//            System.out.println(instance.getActualMaximum(Calendar.WEEK_OF_MONTH));
//        --------------------
            wrappedDataMap.clear();
            indexToRowKey.clear();
            rowKeyToIndex.clear();
            int i = 0;
            for (ScheduleTask task : allTasks) {
                if (task.getStartDate().compareTo(startDate) >= 0 && task.getStartDate().compareTo(endDate) < 0) {
                    wrappedDataMap.put(task.getId(), task);
                    int index = i++;
                    indexToRowKey.put(index, task.getId());
                    rowKeyToIndex.put(task.getId(), index);
                    visitor.process(context, task.getId(), argument);
                }
            }
            rowCount = -1;
        }

        @Override
        public boolean isRowAvailable() {
            if (getRowKey() == null) {
                return false;
            } else {
                return null != wrappedDataMap.get(getRowKey());
            }
        }

        @Override
        public int getRowCount() {
            if (rowCount == -1) {
                rowCount = wrappedDataMap.size();
            }
            return rowCount;
        }

        @Override
        public Object getRowData() {
            if (getRowKey() == null) {
                return null;
            } else {
                return wrappedDataMap.get(getRowKey());
            }
        }

        @Override
        public int getRowIndex() {
            return rowIndex;
        }

        @Override
        public void setRowIndex(int rowIndex) {
            this.rowIndex = rowIndex;
            Object key = indexToRowKey.get(rowIndex);
            if ((key != null && !key.equals(getRowKey())) || (key == null && getRowKey() != null)) {
                setRowKey(key);
            }
        }

        @Override
        public Object getWrappedData() {
            throw new UnsupportedOperationException("Not supported yet.");
        }

        @Override
        public void setWrappedData(Object data) {
            throw new UnsupportedOperationException("Not supported yet.");
        }
    };
}
